@@ -4,7 +4,6 @@ import Menu from '@material-ui/core/Menu';
 import MenuItem from '@material-ui/core/MenuItem';
 
 import BorderInnerIcon from '@material-ui/icons/BorderInner';
-import Brightness2Icon from '@material-ui/icons/Brightness3';
 import CloudQueueIcon from '@material-ui/icons/CloudQueue';
 import ExposureIcon from '@material-ui/icons/Exposure';
 import FontDownloadIcon from '@material-ui/icons/FontDownload';
@@ -14,6 +13,7 @@ import PhotoCameraIcon from '@material-ui/icons/PhotoCamera';
 import PhotoLibraryIcon from '@material-ui/icons/PhotoLibrary';
 import PhotoSizeSelectSmallIcon from '@material-ui/icons/PhotoSizeSelectSmall';
 import PlayArrowIcon from '@material-ui/icons/PlayArrow';
+import SettingsIcon from '@material-ui/icons/Settings';
 import ZoomInIcon from '@material-ui/icons/ZoomIn';
 import ZoomOutIcon from '@material-ui/icons/ZoomOut';
 import React from 'react';
@@ -40,9 +40,6 @@ class ChartToolbar extends React.PureComponent {
         this.props.onShowAxis();
     };
 
-    onDarkMode = () => {
-        this.props.onDarkMode();
-    };
 
     onShowFog = () => {
         this.props.onShowFog();
@@ -53,6 +50,10 @@ class ChartToolbar extends React.PureComponent {
         this.props.onGallery();
     };
 
+    handleMoreOptions = () => {
+        this.props.onMoreOptions();
+    };
+
     setEditSelection = () => {
         this.props.onEditSelection();
     };
@@ -70,7 +71,7 @@ class ChartToolbar extends React.PureComponent {
 
     render() {
 
-        const {dragmode, editSelection, showLabels, showAxis, darkMode} = this.props;
+        const {dragmode, editSelection, showLabels, showAxis} = this.props;
         const {saveImageEl} = this.state;
         return (<React.Fragment>
 
@@ -132,7 +133,7 @@ class ChartToolbar extends React.PureComponent {
             <Tooltip title={"Pan"}>
                 <IconButton edge={false} size={'small'} className={dragmode === 'pan' ? active : ''}
                             aria-label="Pan" onClick={() => this.setDragMode('pan')}>
-                    <svg viewBox="0 0 1000 1000" height="16" width="16">
+                    <svg className={"MuiSvgIcon-root"} viewBox="0 0 1000 1000" height="16" width="16">
                         <path
                             d="m1000 350l-187 188 0-125-250 0 0 250 125 0-188 187-187-187 125 0 0-250-250 0 0 125-188-188 186-187 0 125 252 0 0-250-125 0 187-188 188 188-125 0 0 250 250 0 0-126 187 188z"
                             transform="matrix(1 0 0 -1 0 850)"></path>
@@ -156,7 +157,6 @@ class ChartToolbar extends React.PureComponent {
             </Tooltip>}
 
 
-
             {this.props.is3d && this.props.onShowFog && <Tooltip title={"Show Fog"}>
                 <IconButton edge={false} size={'small'} className={this.props.showFog ? active : ''}
                             aria-label="Show Fog" onClick={() => this.onShowFog()}>
@@ -202,12 +202,14 @@ class ChartToolbar extends React.PureComponent {
 
             </Menu>
 
-            {this.props.onDarkMode && <Tooltip title={"Dark Theme"}>
-                <IconButton edge={false} size={'small'} className={darkMode ? active : ''}
-                            aria-label="Dark Theme" onClick={() => this.onDarkMode()}>
-                    <Brightness2Icon/>
+
+            <Tooltip title={"More Options"}>
+                <IconButton edge={false} size={'small'}
+                            aria-label="More Options" onClick={this.handleMoreOptions}>
+                    <SettingsIcon/>
                 </IconButton>
-            </Tooltip>}
+            </Tooltip>
+
 
             <Tooltip title={"View Gallery"}>
                 <IconButton edge={false} size={'small'}
