@@ -8,7 +8,8 @@ import MenuItem from '@material-ui/core/MenuItem';
 import Paper from '@material-ui/core/Paper';
 import Select from '@material-ui/core/Select';
 import PhotoCameraIcon from '@material-ui/icons/PhotoCamera';
-import {scaleLinear} from 'd3-scale';
+import {scaleLinear, scaleSequential} from 'd3-scale';
+import {interpolateReds} from 'd3-scale-chromatic';
 import React from 'react';
 import ColorSchemeLegend, {drawColorScheme} from './ColorSchemeLegend';
 import {numberFormat} from './formatters';
@@ -60,8 +61,8 @@ class DotPlotCanvas extends React.PureComponent {
                 const node = event.target;
                 var rect = node.getBoundingClientRect();
                 let xy = [event.clientX - rect.left - node.clientLeft, event.clientY - rect.top - node.clientTop];
-                xy[0] /= devicePixelRatio;
-                xy[1] /= devicePixelRatio;
+                // xy[0] /= devicePixelRatio;
+                // xy[1] /= devicePixelRatio;
                 const col = Math.floor((xy[0] - this.size.x) / (maxRadius * 2));
                 const row = Math.floor((xy[1]) / (maxRadius * 2));
 
@@ -263,8 +264,8 @@ class DotPlotCanvas extends React.PureComponent {
                     if (renamed2 != null) {
                         val2 = renamed2;
                     }
-                    val1 = val1.toLowerCase();
-                    val2 = val2.toLowerCase();
+                    val1 = ('' + val1).toLowerCase();
+                    val2 = ('' + val2).toLowerCase();
                     const num1 = parseFloat(val1);
                     const num1Valid = !isNaN(num1);
                     const num2 = parseFloat(val2);
@@ -291,7 +292,7 @@ class DotPlotCanvas extends React.PureComponent {
         this.features = features;
         this.size = this.getSize(context);
         this.categoryOrder = categoryOrder;
-        this.colorScale = scaleLinear().domain(this.props.meanRange).range(['blue', 'red']);
+        this.colorScale = scaleSequential(interpolateReds).domain(this.props.meanRange).clamp(true);
         this.sizeScale = scaleLinear().domain(this.props.fractionRange).range([minRadius, maxRadius]).clamp(true);
     }
 
