@@ -20,6 +20,10 @@ import {getChartSize} from './util';
 
 class EmbeddingChart extends React.PureComponent {
 
+    setChartRef = (element) => {
+        this.props.chartOptions.ref = element;
+    };
+
 
     render() {
         const {
@@ -75,6 +79,8 @@ class EmbeddingChart extends React.PureComponent {
                                    unselectedMarkerOpacity={this.props.unselectedMarkerOpacity}
                                    color={traceInfo.colors}
                                    onGallery={this.props.onGallery}
+                                   onMoreOptions={this.props.onMoreOptions}
+                                   ref={this.setChartRef}
 
                 />}
 
@@ -89,9 +95,11 @@ class EmbeddingChart extends React.PureComponent {
                     onInitialized={this.onInitialized}
                     markerOpacity={this.props.markerOpacity}
                     unselectedMarkerOpacity={this.props.unselectedMarkerOpacity}
-                    onGallery={this.props.onGallery}
                     onDeselect={this.props.onDeselect}
                     onSelected={this.props.onSelect}
+                    onGallery={this.props.onGallery}
+                    onMoreOptions={this.props.onMoreOptions}
+                    ref={this.setChartRef}
                 />}
 
 
