@@ -1,9 +1,9 @@
+import {Typography} from '@material-ui/core';
 import withStyles from '@material-ui/core/styles/withStyles';
 import {bind, uniqueId} from 'lodash';
 import OpenSeadragon from 'openseadragon';
 import React from 'react';
 import simplify from 'simplify-js';
-import {Color} from 'three';
 import {getEmbeddingKey} from './actions';
 import CanvasOverlayHd from './CanvasOverlayHd';
 import ChartToolbar from './ChartToolbar';
@@ -33,7 +33,7 @@ function drawLabels(context, zoom, traceInfo, showLabels, categoricalNames) {
     showLabels = showLabels && traceInfo.isCategorical;
     if (showLabels) {
         context.fillStyle = 'black';
-        const fontSize = Math.ceil(18 * 1 / zoom);
+        const fontSize = Math.ceil(this.props.chartOptions.labelFontSize * 1 / zoom);
         context.font = fontSize + 'px Roboto Condensed,Helvetica,Arial,sans-serif';
         context.textAlign = 'center';
         const labelsPositions = getCategoryLabelsPositions(traceInfo, categoricalNames);
@@ -138,10 +138,17 @@ class ImageChart extends React.PureComponent {
     }
 
     setTooltip(xpix, ypix) {
-        let data = this.props.traceInfo;
+        let traceInfo = this.props.traceInfo;
         const point = this.findPointIndex(xpix, ypix);
         if (point != -1) {
-            let value = data.values[point];
+            let value = traceInfo.values[point];
+            let categoryObject = this.props.categoricalNames[traceInfo.name];
+            if (categoryObject) {
+                let renamedValue = categoryObject[value];
+                if (renamedValue != null) {
+                    value = renamedValue;
+                }
+            }
             if (typeof value === 'number') {
                 value = numberFormat(value);
             }
@@ -422,14 +429,10 @@ class ImageChart extends React.PureComponent {
     };
 
     onDragMode = (mode) => {
-        this.props.chartOptions.dragmode = mode
+        this.props.chartOptions.dragmode = mode;
         this.props.setChartOptions(this.props.chartOptions);
     };
 
-    onDarkMode = () => {
-        this.props.chartOptions.darkMode = !this.props.chartOptions.darkMode;
-        this.props.setChartOptions(this.props.chartOptions);
-    };
 
     render() {
 
@@ -441,8 +444,7 @@ class ImageChart extends React.PureComponent {
                     editSelection={this.props.chartOptions.editSelection}
                     onZoomIn={this.onZoomIn}
                     onZoomOut={this.onZoomOut}
-                    darkMode={this.props.chartOptions.darkMode}
-                    onDarkMode={this.onDarkMode}
+                    onMoreOptions={this.onMoreOptions}
                     showLabels={this.props.chartOptions.showLabels}
                     is3d={false}
                     onHome={this.onHome}
@@ -452,11 +454,11 @@ class ImageChart extends React.PureComponent {
                     onEditSelection={this.onEditSelection}
                     onGallery={this.onGallery}>
                 </ChartToolbar>
-                <div ref={this.tooltipElementRef} style={{
+                <Typography color="textPrimary" ref={this.tooltipElementRef} style={{
                     display: 'inline-block',
                     paddingLeft: 5,
                     verticalAlign: 'top'
-                }}>&nbsp;</div>
+                }}>&nbsp;</Typography>
             </div>
 
             <div style={{
