@@ -1,13 +1,14 @@
+import {Typography} from '@material-ui/core';
 import withStyles from '@material-ui/core/styles/withStyles';
 import {scaleLinear} from 'd3-scale';
 import {bind} from 'lodash';
 import React from 'react';
-import {Color, Vector3, Vector4} from 'three';
+import {Vector3, Vector4} from 'three';
 import {getEmbeddingKey} from './actions';
 import ChartToolbar from './ChartToolbar';
 import {saveImage} from './ChartUtil';
 import {numberFormat} from './formatters';
-import {createScatterPlot, getCategoryLabelsPositions, updateScatterChart} from './ThreeUtil';
+import {createScatterPlot, getCategoryLabelsPositions, POINT_VISUALIZER_ID, updateScatterChart} from './ThreeUtil';
 import {getChartSize, isPointInside} from './util';
 
 function clamp(x, min_v, max_v) {
@@ -23,9 +24,10 @@ function mix(x, y, a) {
     return x * (1.0 - a) + y * a;
 }
 
-export function getPointVisualizer(scatterPlot) {
+
+export function getVisualizer(scatterPlot, id) {
     for (let i = 0; i < scatterPlot.visualizers.length; i++) {
-        if (scatterPlot.visualizers[i].id === 'SPRITES') {
+        if (scatterPlot.visualizers[i].id === id) {
             return scatterPlot.visualizers[i];
         }
     }
@@ -66,6 +68,7 @@ class ScatterChartThree extends React.PureComponent {
         this.tooltipElementRef = React.createRef();
         this.scatterPlot = null;
         this.chartSize = getChartSize();
+        this.lastHoverIndex = -1;
         // window.addEventListener('resize', () => {
         //     scatterGL.resize();
         // });
@@ -78,8 +81,14 @@ class ScatterChartThree extends React.PureComponent {
 
     componentDidMount() {
         this.init();
+
         this.draw(); // TODO fix 2x draw
         this.draw();
+        if (this.props.chartOptions.camera) {
+            this.scatterPlot.updateFromCameraDef(this.props.chartOptions.camera);
+            this.props.chartOptions.camera = null;
+            this.draw();
+        }
     }
 
     componentWillUnmount() {
@@ -111,6 +120,10 @@ class ScatterChartThree extends React.PureComponent {
         const camera = this.scatterPlot.camera;
         const width = chartSize.width;
         const height = chartSize.height;
+        if (chartOptions.darkMode) {
+            context.fillStyle = 'black';
+            context.fillRect(0, 0, width, height);
+        }
         const widthHalf = width / 2;
         const heightHalf = height / 2;
         const colorScale = scaleLinear().domain([0, 1]).range([0, 255]);
@@ -119,7 +132,7 @@ class ScatterChartThree extends React.PureComponent {
         const is3d = traceInfo.dimensions === 3;
         let outputPointSize = 0;
         let fog = this.scatterPlot.scene.fog;
-        let spriteVisualizer = getPointVisualizer(this.scatterPlot);
+        let spriteVisualizer = getVisualizer(this.scatterPlot, POINT_VISUALIZER_ID);
         if (!is3d) {
             const PI = 3.1415926535897932384626433832795;
             const minScale = 0.1;  // minimum scaling factor
@@ -188,11 +201,14 @@ class ScatterChartThree extends React.PureComponent {
             context.fill();
         }
         if (showLabels) {
-            context.fillStyle = 'black';
-
+            context.fillStyle = chartOptions.darkMode ? 'white' : 'black';
+            context.strokeStyle = chartOptions.darkMode ? 'rgba(0,0,0,0.9)' : 'rgba(255,255,255,0.9)';
+            context.strokeWidth = chartOptions.labelStrokeWidth;
             let font = format === 'svg' ? 'serif' : 'Roboto Condensed';
-            context.font = '24px ' + font; // FIXME adjust size
+            context.font = 'bold ' + this.props.chartOptions.labelFontSize + 'px ' + font;
             const labelsPositions = getCategoryLabelsPositions(traceInfo, categoricalNames);
+            context.textAlign = 'center';
+            context.textBaseline = "middle";
             for (let i = 0, k = 0; i < labelsPositions.labels.length; i++, k += 3) {
                 pos.x = labelsPositions.positions[k];
                 pos.y = labelsPositions.positions[k + 1];
@@ -200,8 +216,9 @@ class ScatterChartThree extends React.PureComponent {
                 pos.project(camera);
                 pos.x = (pos.x * widthHalf) + widthHalf;
                 pos.y = -(pos.y * heightHalf) + heightHalf;
-                context.fillText(labelsPositions.labels[i], pos.x, pos.y);
 
+                context.strokeText(labelsPositions.labels[i], pos.x, pos.y);
+                context.fillText(labelsPositions.labels[i], pos.x, pos.y);
             }
         }
     }
@@ -226,6 +243,11 @@ class ScatterChartThree extends React.PureComponent {
         saveImage(traceInfo, this.chartSize, bind(this.drawContext, this), format);
     };
 
+    onMoreOptions = () => {
+        this.props.onMoreOptions();
+    };
+
+
     onEditSelection = () => {
         this.props.chartOptions.editSelection = !this.props.chartOptions.editSelection;
         this.props.setChartOptions(this.props.chartOptions);
@@ -237,11 +259,6 @@ class ScatterChartThree extends React.PureComponent {
         this.props.setChartOptions(this.props.chartOptions);
     };
 
-    onDarkMode = () => {
-        this.props.chartOptions.darkMode = !this.props.chartOptions.darkMode;
-        this.scatterPlot.scene.background = this.props.chartOptions.darkMode ? new Color("rgb(0, 0, 0)") : null;
-        this.props.setChartOptions(this.props.chartOptions);
-    };
 
     onGallery = () => {
         this.props.onGallery();
@@ -258,7 +275,7 @@ class ScatterChartThree extends React.PureComponent {
     };
 
     onShowFog = () => {
-        let spriteVisualizer = getPointVisualizer(this.scatterPlot);
+        let spriteVisualizer = getVisualizer(this.scatterPlot, POINT_VISUALIZER_ID);
         this.props.chartOptions.showFog = !this.props.chartOptions.showFog;
         spriteVisualizer.styles.fog.enabled = this.props.chartOptions.showFog;
         this.props.setChartOptions(this.props.chartOptions);
@@ -266,6 +283,7 @@ class ScatterChartThree extends React.PureComponent {
 
 
     onDragMode = (mode) => {
+
         if (mode === 'pan') {
             this.scatterPlot.setInteractionMode('PAN');
         } else if (mode === 'select') {
@@ -295,23 +313,80 @@ class ScatterChartThree extends React.PureComponent {
 
         if (this.scatterPlot == null) {
             const containerElement = this.containerElementRef.current;
-            this.scatterPlot = createScatterPlot(containerElement, false);
+            this.scatterPlot = createScatterPlot(containerElement, false, true);
+            if (this.props.chartOptions.dragmode === 'pan') {
+                this.scatterPlot.setInteractionMode('PAN');
+            } else if (this.props.chartOptions.dragmode === 'select') {
+                this.scatterPlot.setInteractionMode('SELECT');
+                this.scatterPlot.rectangleSelector.setSelectionMode('BOX');
+            } else if (this.props.chartOptions.dragmode === 'lasso') {
+                this.scatterPlot.setInteractionMode('SELECT');
+                this.scatterPlot.rectangleSelector.setSelectionMode('LASSO');
+
+            }
             const axes = this.scatterPlot.scene.getObjectByName('axes');
             if (axes) {
                 axes.visible = this.props.chartOptions.showAxis;
             }
-            let spriteVisualizer = getPointVisualizer(this.scatterPlot);
+            let spriteVisualizer = getVisualizer(this.scatterPlot, POINT_VISUALIZER_ID);
             spriteVisualizer.styles.fog.enabled = this.props.chartOptions.showFog;
             this.scatterPlot.hoverCallback = (point) => {
                 if (point == null) {
                     this.tooltipElementRef.current.innerHTML = ' ';
                 } else {
                     const traceInfo = this.props.traceInfo;
-                    let value = traceInfo.values[point];
-                    if (typeof value === 'number') {
-                        value = numberFormat(value);
+                    const positions = traceInfo.positions;
+                    const camera = this.scatterPlot.camera;
+                    const widthHalf = this.chartSize.width / 2;
+                    const heightHalf = this.chartSize.height / 2;
+                    const pos = new Vector3();
+                    let selectedIndex = -1;
+                    const tolerance = 2;
+                    if (this.lastHoverIndex !== -1) {
+                        pos.x = positions[this.lastHoverIndex * 3];
+                        pos.y = positions[this.lastHoverIndex * 3 + 1];
+                        pos.z = positions[this.lastHoverIndex * 3 + 2];
+                        pos.project(camera);
+                        pos.x = (pos.x * widthHalf) + widthHalf;
+                        pos.y = -(pos.y * heightHalf) + heightHalf;
+                        if (Math.abs(pos.x - point.x) <= tolerance && Math.abs(pos.y - point.y) <= tolerance) {
+                            selectedIndex = this.lastHoverIndex;
+                        }
                     }
-                    this.tooltipElementRef.current.innerHTML = '' + value;
+
+                    if (selectedIndex === -1) {
+                        for (let i = 0, j = 0, k = 0; i < traceInfo.npoints; i++, j += 4, k += 3) {
+                            pos.x = positions[k];
+                            pos.y = positions[k + 1];
+                            pos.z = positions[k + 2];
+                            pos.project(camera);
+                            pos.x = (pos.x * widthHalf) + widthHalf;
+                            pos.y = -(pos.y * heightHalf) + heightHalf;
+                            if (Math.abs(pos.x - point.x) <= tolerance && Math.abs(pos.y - point.y) <= tolerance) {
+                                selectedIndex = i;
+                                break;
+                            }
+                        }
+                    }
+                    this.lastHoverIndex = selectedIndex;
+                    if (selectedIndex !== -1) {
+                        let value = traceInfo.values[selectedIndex];
+                        let categoryObject = this.props.categoricalNames[traceInfo.name];
+                        if (categoryObject) {
+                            let renamedValue = categoryObject[value];
+                            if (renamedValue != null) {
+                                value = renamedValue;
+                            }
+                        }
+
+                        if (typeof value === 'number') {
+                            value = numberFormat(value);
+                        }
+                        this.tooltipElementRef.current.innerHTML = '' + value;
+                    } else {
+                        this.tooltipElementRef.current.innerHTML = ' ';
+                    }
+
                 }
 
             };
@@ -319,11 +394,8 @@ class ScatterChartThree extends React.PureComponent {
                 const traceInfo = this.props.traceInfo;
                 const positions = traceInfo.positions;
                 const camera = this.scatterPlot.camera;
-                const chartSize = this.chartSize;
-                const width = chartSize.width;
-                const height = chartSize.height;
-                const widthHalf = width / 2;
-                const heightHalf = height / 2;
+                const widthHalf = this.chartSize.width / 2;
+                const heightHalf = this.chartSize.height / 2;
                 const pos = new Vector3();
                 const selectedPoints = [];
 
@@ -338,6 +410,7 @@ class ScatterChartThree extends React.PureComponent {
                         selectedPoints.push(i);
                     }
                 }
+
                 if (selectedPoints.length === 0) {
                     this.props.onDeselect({name: getEmbeddingKey(traceInfo.embedding)});
                 } else {
@@ -348,81 +421,53 @@ class ScatterChartThree extends React.PureComponent {
                     });
                 }
             };
-            this.scatterPlot.selectCallback = (selectedpoints) => {
+            this.scatterPlot.boxCallback = (rect) => {
                 if (this.scatterPlot.interactionMode === 'PAN') {
                     return;
                 }
-                if (selectedpoints != null && selectedpoints.length === 0) {
-                    selectedpoints = null;
-                }
                 const traceInfo = this.props.traceInfo;
-                if (selectedpoints == null) {
+                const positions = traceInfo.positions;
+                const camera = this.scatterPlot.camera;
+                const widthHalf = this.chartSize.width / 2;
+                const heightHalf = this.chartSize.height / 2;
+                const pos = new Vector3();
+                const selectedPoints = [];
+
+                for (let i = 0, j = 0, k = 0; i < traceInfo.npoints; i++, j += 4, k += 3) {
+                    pos.x = positions[k];
+                    pos.y = positions[k + 1];
+                    pos.z = positions[k + 2];
+                    pos.project(camera);
+                    pos.x = (pos.x * widthHalf) + widthHalf;
+                    pos.y = -(pos.y * heightHalf) + heightHalf;
+                    if (pos.x >= rect.x && pos.x <= (rect.x + rect.width) && pos.y >= rect.y && pos.y <= (rect.y + rect.height)) {
+                        selectedPoints.push(i);
+                    }
+                }
+
+                if (selectedPoints.length === 0) {
                     this.props.onDeselect({name: getEmbeddingKey(traceInfo.embedding)});
                 } else {
-                    // let xmin = Number.MAX_VALUE;
-                    // let ymin = Number.MAX_VALUE;
-                    // let zmin = Number.MAX_VALUE;
-                    // let xmax = -Number.MAX_VALUE;
-                    // let ymax = -Number.MAX_VALUE;
-                    // let zmax = -Number.MAX_VALUE;
-                    //
-                    // const is3d = traceInfo.dimensions === 3;
-                    // selectedpoints.forEach(index => {
-                    //     const x = traceInfo.x[index];
-                    //     xmin = Math.min(xmin, x);
-                    //     xmax = Math.max(xmax, x);
-                    //     const y = traceInfo.y[index];
-                    //     ymin = Math.min(ymin, y);
-                    //     ymax = Math.max(ymax, y);
-                    //     if (is3d) {
-                    //         const z = traceInfo.z[index];
-                    //         zmin = Math.min(zmin, z);
-                    //         zmax = Math.max(zmax, z);
-                    //     }
-                    // });
-                    //
-                    //
-                    // let path = {
-                    //     shape: 'rect',
-                    //     x: xmin,
-                    //     y: ymin,
-                    //     width: xmax - xmin,
-                    //     height: ymax - ymin
-                    // };
-                    // if (is3d) {
-                    //     path.shape = 'rect 3d';
-                    //     if (zmin > zmax) {
-                    //         const tmp = zmin;
-                    //         zmin = zmax;
-                    //         zmax = tmp;
-                    //     }
-                    //     path.z = zmin;
-                    //     path.depth = zmax - zmin;
-                    // }
-                    //
-                    // this.props.onSelected({
-                    //     name: getEmbeddingKey(traceInfo.embedding),
-                    //     clear: !this.props.chartOptions.editSelection,
-                    //     value: {basis: traceInfo.embedding, path: path}
-                    // });
                     this.props.onSelected({
                         name: getEmbeddingKey(traceInfo.embedding),
                         clear: !this.props.chartOptions.editSelection,
-                        value: {basis: traceInfo.embedding, points: selectedpoints}
+                        value: {basis: traceInfo.embedding, points: selectedPoints}
                     });
                 }
             };
-
-
-            const canvas = this.containerElementRef.current.querySelector('canvas');
-            canvas.style.outline = '0px';
         }
+        ;
+
+
+        const canvas = this.containerElementRef.current.querySelector('canvas');
+        canvas.style.outline = '0px';
     }
 
+
     draw() {
         const {traceInfo, markerOpacity, unselectedMarkerOpacity, selection, pointSize, chartOptions, categoricalNames} = this.props;
         updateScatterChart(this.scatterPlot, traceInfo, selection, markerOpacity, unselectedMarkerOpacity, pointSize,
-            chartOptions.showLabels, categoricalNames, chartOptions.showFog, chartOptions.showAxis, chartOptions.darkMode);
+            categoricalNames, chartOptions);
     }
 
 
@@ -434,10 +479,9 @@ class ScatterChartThree extends React.PureComponent {
                     animating={this.props.chartOptions.animating}
                     editSelection={this.props.chartOptions.editSelection}
                     showLabels={this.props.chartOptions.showLabels}
-                    darkMode={this.props.chartOptions.darkMode}
-                    onDarkMode={this.onDarkMode}
                     showFog={this.props.chartOptions.showFog}
                     onShowFog={this.onShowFog}
+                    onMoreOptions={this.onMoreOptions}
                     is3d={this.props.traceInfo && this.props.traceInfo.z != null}
                     toggleAnimation={this.onToggleAnimation}
                     onSaveImage={this.onSaveImage}
@@ -449,11 +493,11 @@ class ScatterChartThree extends React.PureComponent {
                     showAxis={this.props.chartOptions.showAxis}
                     onGallery={this.onGallery}>
                 </ChartToolbar>
-                <div ref={this.tooltipElementRef} style={{
+                <Typography color="textPrimary" ref={this.tooltipElementRef} style={{
                     display: 'inline-block',
                     paddingLeft: 5,
                     verticalAlign: 'top'
-                }}>&nbsp;</div>
+                }}>&nbsp;</Typography>
             </div>
 
             <div style={{
