@@ -1,7 +1,7 @@
 import {color} from 'd3-color';
-import {makeStyles, ScatterPlot, ScatterPlotVisualizer3DLabels, ScatterPlotVisualizerSprites} from 'scatter-gl';
+import {makeStyles, ScatterPlot, ScatterPlotVisualizerSprites, ScatterPlotVisualizerSvgLabels} from 'scatter-gl';
 import {Color} from 'three';
-import {getPointVisualizer} from './ScatterChartThree';
+import {getVisualizer} from './ScatterChartThree';
 import {getRgbScale} from './util';
 
 function scaleLinear(value, domain, range) {
@@ -12,8 +12,11 @@ function scaleLinear(value, domain, range) {
     return percentDomain * rangeDifference + range[0];
 }
 
+export const POINT_VISUALIZER_ID = 'SPRITES';
 
-export function createScatterPlot(containerElement, premultipliedAlpha) {
+export const LABELS_VISUALIZER_ID = 'SVG_LABELS';
+
+export function createScatterPlot(containerElement, premultipliedAlpha, labels) {
     const styles = makeStyles();
     styles.label3D.fontSize = 40;
 
@@ -23,11 +26,12 @@ export function createScatterPlot(containerElement, premultipliedAlpha) {
         styles: styles,
 
     }, premultipliedAlpha); // toDataUrl images are flipped on Safari when premultipliedAlpha is false
-
-    const activeVisualizers = [];
-    const visualizer = new ScatterPlotVisualizerSprites(styles);
-    activeVisualizers.push(visualizer);
-    scatterPlot.setActiveVisualizers(activeVisualizers);
+    let visualizers = [new ScatterPlotVisualizerSprites(styles)];
+    if (labels) {
+        // visualizers.push(new ScatterPlotVisualizerCanvasLabels(containerElement, styles));
+        visualizers.push(new ScatterPlotVisualizerSvgLabels(containerElement, styles));
+    }
+    scatterPlot.setActiveVisualizers(visualizers);
     return scatterPlot;
 }
 
@@ -113,18 +117,18 @@ export function getCategoryLabelsPositions(traceInfo, categoricalNames) {
         let value = traceInfo.values[i];
         let p = categoryToPosition[value];
         if (p === undefined) {
-            p = {count: 0, position: [0, 0, 0]};
+            p = {x: [], y: [], z: []};
             categoryToPosition[value] = p;
             ncategories++;
         }
         p.count++;
         if (isImage) {
-            p.position[0] += traceInfo.x[i];
-            p.position[1] += traceInfo.y[i];
+            p.x.push(traceInfo.x[i]);
+            p.y.push(traceInfo.y[i]);
         } else {
-            p.position[0] += traceInfo.positions[j];
-            p.position[1] += traceInfo.positions[j + 1];
-            p.position[2] += traceInfo.positions[j + 2];
+            p.x.push(traceInfo.positions[j]);
+            p.y.push(traceInfo.positions[j + 1]);
+            p.z.push(traceInfo.positions[j + 2]);
         }
     }
     let labelStrings = [];
@@ -142,16 +146,20 @@ export function getCategoryLabelsPositions(traceInfo, categoricalNames) {
             labelStrings.push(category);
         }
         let p = categoryToPosition[category];
-        labelPositions[positionIndex] = p.position[0] / p.count;
-        labelPositions[positionIndex + 1] = p.position[1] / p.count;
-        labelPositions[positionIndex + 2] = p.position[2] / p.count;
+        p.x.sort((a, b) => a - b);
+        p.y.sort((a, b) => a - b);
+        p.z.sort((a, b) => a - b);
+        const mid = p.x.length / 2;
+        labelPositions[positionIndex] = mid % 1 ? p.x[mid - 0.5] : (p.x[mid - 1] + p.x[mid]) / 2;
+        labelPositions[positionIndex + 1] = mid % 1 ? p.y[mid - 0.5] : (p.y[mid - 1] + p.y[mid]) / 2;
+        labelPositions[positionIndex + 2] = mid % 1 ? p.z[mid - 0.5] : (p.z[mid - 1] + p.z[mid]) / 2;
         positionIndex += 3;
     }
 
     return {labels: labelStrings, positions: labelPositions};
 }
 
-export function updateScatterChart(scatterPlot, traceInfo, selection, markerOpacity, unselectedMarkerOpacity, pointSize, showLabels = false, categoricalNames = {}, showFog = true, showAxis = true, darkMode = false) {
+export function updateScatterChart(scatterPlot, traceInfo, selection, markerOpacity, unselectedMarkerOpacity, pointSize, categoricalNames = {}, chartOptions) {
     const colors = traceInfo.colors;
     const positions = traceInfo.positions;
     const is3d = traceInfo.z != null;
@@ -162,13 +170,13 @@ export function updateScatterChart(scatterPlot, traceInfo, selection, markerOpac
             positions[k] = isSelected ? 1 : 0;
         }
     }
-    scatterPlot.scene.background = darkMode ? new Color("rgb(0, 0, 0)") : null;
+    scatterPlot.scene.background = chartOptions.darkMode ? new Color("rgb(0, 0, 0)") : null;
     scatterPlot.setDimensions(traceInfo.dimensions);
-    let spriteVisualizer = getPointVisualizer(scatterPlot);
-    spriteVisualizer.styles.fog.enabled = showFog;
+    let spriteVisualizer = getVisualizer(scatterPlot, POINT_VISUALIZER_ID);
+    spriteVisualizer.styles.fog.enabled = chartOptions.showFog;
     const axes = scatterPlot.scene.getObjectByName('axes');
     if (axes) {
-        axes.visible = showAxis;
+        axes.visible = chartOptions.showAxis;
     }
     scatterPlot.setPointColors(colors);
     scatterPlot.setPointPositions(positions);
@@ -179,17 +187,19 @@ export function updateScatterChart(scatterPlot, traceInfo, selection, markerOpac
     scale.fill(pointSize);
     scatterPlot.setPointScaleFactors(scale);
 
-    showLabels = showLabels && traceInfo.isCategorical;
-
-    let activeVisualizers = scatterPlot.getActiveVisualizers();
-    activeVisualizers = activeVisualizers.filter(vis => !(vis instanceof ScatterPlotVisualizer3DLabels));
-    if (showLabels) {
-        const labelsPositions = getCategoryLabelsPositions(traceInfo, categoricalNames);
-        let labels3DVisualizer = new ScatterPlotVisualizer3DLabels(scatterPlot.styles);
-        labels3DVisualizer.setLabels(labelsPositions.labels, labelsPositions.positions);
-        activeVisualizers.push(labels3DVisualizer);
+    const showLabels = chartOptions.showLabels && traceInfo.isCategorical;
+    const labelsVisualizer = getVisualizer(scatterPlot, LABELS_VISUALIZER_ID);
+    if (labelsVisualizer) {
+        labelsVisualizer.labelsActive = showLabels;
+        if (showLabels) {
+            const labelsPositions = getCategoryLabelsPositions(traceInfo, categoricalNames);
+            labelsVisualizer.fillStyle = chartOptions.darkMode ? 'white' : 'black';
+            labelsVisualizer.shadowColor = chartOptions.darkMode ? 'rgba(0,0,0,0.9)' : 'rgba(255,255,255,0.9)';
+            labelsVisualizer.shadowStroke = chartOptions.labelStrokeWidth;
+            labelsVisualizer.setLabels(labelsPositions.labels, labelsPositions.positions);
+            labelsVisualizer.font = 'bold ' + chartOptions.labelFontSize + 'px Roboto Condensed';
+        }
     }
 
-    scatterPlot.setActiveVisualizers(activeVisualizers);
     scatterPlot.render();
 }
\ No newline at end of file
