@@ -3,6 +3,25 @@ import {schemeCategory10, schemePaired} from 'd3-scale-chromatic';
 import {saveAs} from 'file-saver';
 import {isEqual, isPlainObject, uniqBy} from 'lodash';
 import OpenSeadragon from 'openseadragon';
+import {
+    deleteDatasetFilterPromise,
+    exportDatasetFiltersPromise,
+    getCategoryNamesPromise,
+    getDatasetFilterPromise,
+    getDatasetsPromise,
+    getEmbeddingPromise,
+    getFileUrl,
+    getFiltersPromise,
+    getGroupedStatsPromise,
+    getSchemaPromise,
+    getSelectedIdsPromise,
+    getSelectionPromise,
+    getServerPromise,
+    getStatsPromise,
+    setCategoryNamePromise,
+    upsertDatasetFilterPromise
+} from '../api_util';
+
 import CustomError from '../CustomError';
 import {getPositions} from '../ThreeUtil';
 
@@ -16,7 +35,6 @@ import {
 } from '../util';
 
 // export const API = 'http://localhost:5000/api';
-
 export const API = '/api';
 
 const authScopes = [
@@ -39,7 +57,7 @@ export const ADD_DATASET = 'ADD_DATASET';
 export const DELETE_DATASET = 'DELETE_DATASET';
 export const UPDATE_DATASET = 'UPDATE_DATASET';
 export const SET_GLOBAL_FEATURE_SUMMARY = 'SET_GLOBAL_FEATURE_SUMMARY';
-export const DIFF_EXP_RESULTS = 'DIFF_EXP_RESULTS';
+export const SET_SAVED_DATASET_STATE = 'SET_SAVED_DATASET_STATE';
 
 export const SET_CATEGORICAL_COLOR = 'SET_CATEGORICAL_COLOR';
 export const SET_CATEGORICAL_NAME = 'SET_CATEGORICAL_NAME';
@@ -72,6 +90,7 @@ export const SET_DATASET = 'SET_DATASET';
 
 export const SET_DIALOG = 'SET_DIALOG';
 
+export const MORE_OPTIONS_DIALOG = 'MORE_OPTIONS_DIALOG';
 export const EDIT_DATASET_DIALOG = 'EDIT_DATASET_DIALOG';
 export const IMPORT_DATASET_DIALOG = 'IMPORT_DATASET_DIALOG';
 export const SAVE_DATASET_FILTER_DIALOG = 'SAVE_DATASET_FILTER_DIALOG';
@@ -153,13 +172,13 @@ export function initGapi() {
 
         window.setTimeout(loadingAppProgress, 500);
 
-        fetch(API + '/server').then(result => result.json()).then(serverInfo => {
+        getServerPromise().then(serverInfo => {
             dispatch(setServerInfo(serverInfo));
-            if (serverInfo.clientId ==='') { // no login required
+            if (serverInfo.clientId === '') { // no login required
                 dispatch(_setLoadingApp({loading: false}));
                 dispatch(_setEmail(serverInfo.mode === 'server' ? '' : null));
-                if(serverInfo.mode === 'server') {
-                    dispatch(setUser({importer:true}))
+                if (serverInfo.mode === 'server') {
+                    dispatch(setUser({importer: true}));
                 }
                 dispatch(listDatasets()).then(() => {
                     dispatch(_loadSavedView());
@@ -211,10 +230,7 @@ export function openDatasetFilter(filterId) {
 
         dispatch(_setLoading(true));
 
-        fetch(API + '/filter?id=' + filterId,
-            {
-                headers: {'Authorization': 'Bearer ' + getIdToken()},
-            }).then(response => response.json())
+        getDatasetFilterPromise(filterId, getState().dataset.id)
             .then(result => {
                 result.id = filterId;
                 let filterValue = JSON.parse(result.value);
@@ -248,12 +264,7 @@ export function openDatasetFilter(filterId) {
 export function deleteDatasetFilter(filterId) {
     return function (dispatch, getState) {
         dispatch(_setLoading(true));
-        fetch(API + '/filter',
-            {
-                body: JSON.stringify({id: filterId}),
-                method: 'DELETE',
-                headers: {'Authorization': 'Bearer ' + getIdToken()},
-            })
+        deleteDatasetFilterPromise(filterId, getState().dataset.id)
             .then(() => {
                 let datasetFilters = getState().datasetFilters;
                 for (let i = 0; i < datasetFilters.length; i++) {
@@ -307,12 +318,7 @@ export function saveDatasetFilter(payload) {
             savedDatasetFilter[key] = requestBody[key];
         }
         dispatch(_setLoading(true));
-        fetch(API + '/filter',
-            {
-                body: JSON.stringify(requestBody),
-                method: payload.filterId != null ? 'PUT' : 'POST',
-                headers: {'Authorization': 'Bearer ' + getIdToken()},
-            }).then(response => response.json())
+        upsertDatasetFilterPromise(requestBody, payload.filterId != null)
             .then(result => {
                 requestBody.id = result.id;
                 let datasetFilters = getState().datasetFilters;
@@ -408,64 +414,14 @@ function getFilterJson(state) {
 }
 
 
-export function diffExp() {
-    return function (dispatch, getState) {
-        dispatch(_setLoading(true));
-        let filter = getFilterJson(getState(), true);
-        let nfeatures = getState().dataset.features.length;
-        let batchSize = 3000; // TODO
-        let promises = [];
-        let results = null;
-        let start = 0;
-        let end = batchSize;
-        let batches = Math.ceil(nfeatures / batchSize);
-        for (let i = 0; i < batches; i++) {
-            end = Math.min(end + batchSize, nfeatures);
-            start = Math.min(start + batchSize, nfeatures);
-            if (end > start) {
-                let p = fetch(API + '/diff_exp',
-                    {
-                        body: JSON.stringify({id: getState().dataset.id, filter: filter, var_range: [start, end]}),
-                        method: 'POST',
-                        headers: {'Content-Type': 'application/json', 'Authorization': 'Bearer ' + getIdToken()},
-                    }).then(result => result.json()).then(result => {
-                    if (results == null) {
-                        results = result;
-                    } else {
-                        for (let key in result) {
-                            results[key] = results[key].concat(result[key]);
-                        }
-                    }
-                });
-                promises.push(p);
-            }
-        }
-
-        Promise.all(promises).then(() => {
-            let keys = Object.keys(results);
-            let indices = new Uint16Array(results[keys[0]].length); // for sort order
-            for (let i = 0, n = indices.length; i < n; i++) {
-                indices[i] = i;
-            }
-            dispatch(setDiffExpResults({arrays: results, indices: indices}));
-        }).finally(() => {
-            dispatch(_setLoading(false));
-        }).catch(err => {
-            handleError(dispatch, err);
-        });
-    };
-}
-
 export function downloadSelectedIds() {
     return function (dispatch, getState) {
         dispatch(_setLoading(true));
         let filter = getFilterJson(getState(), true);
-        fetch(API + '/selected_ids',
-            {
-                body: JSON.stringify({id: getState().dataset.id, filter: filter}),
-                method: 'POST',
-                headers: {'Content-Type': 'application/json', 'Authorization': 'Bearer ' + getIdToken()},
-            }).then(result => result.json()).then(result => {
+        getSelectedIdsPromise({
+            id: getState().dataset.id,
+            filter: filter
+        }).then(result => result.json()).then(result => {
             const blob = new Blob([result.ids.join('\n')], {type: "text/plain;charset=utf-8"});
             saveAs(blob, "selection.txt");
         }).finally(() => {
@@ -479,15 +435,11 @@ export function downloadSelectedIds() {
 export function exportDatasetFilters() {
     return function (dispatch, getState) {
         dispatch(_setLoading(true));
-        fetch(API + '/export_filters?id=' + getState().dataset.id, {
-            headers: {'Authorization': 'Bearer ' + getIdToken()},
-        }).then(result => {
-            if (!result.ok) {
+        exportDatasetFiltersPromise(getState().dataset.id).then(result => {
+            if (result == null) {
                 handleError(dispatch, 'Unable to export filters');
                 return;
             }
-            return result.text();
-        }).then(result => {
             const blob = new Blob([result], {type: "text/plain;charset=utf-8"});
             saveAs(blob, "filters.csv");
         }).finally(() => {
@@ -519,10 +471,6 @@ export function setCombineDatasetFilters(payload) {
     };
 }
 
-export function setDiffExpResults(payload) {
-    return {type: DIFF_EXP_RESULTS, payload: payload};
-}
-
 export function setChartSize(payload) {
     return {type: SET_CHART_SIZE, payload: payload};
 }
@@ -587,12 +535,7 @@ function handleFilterUpdated() {
             return getEmbeddingJson(e);
         });
 
-        fetch(API + '/selection',
-            {
-                body: JSON.stringify(json),
-                method: 'POST',
-                headers: {'Content-Type': 'application/json', 'Authorization': 'Bearer ' + getIdToken()},
-            }).then(result => result.json()).then(result => {
+        getSelectionPromise(json).then(result => {
             dispatch(handleSelectionResult(result, true));
         }).catch(err => {
             handleError(dispatch, err);
@@ -600,6 +543,7 @@ function handleFilterUpdated() {
     };
 }
 
+
 export function handleBrushFilterUpdated(payload) {
     return function (dispatch, getState) {
         const name = payload.name; // full basis name
@@ -679,12 +623,12 @@ function _handleCategoricalNameChange(payload) {
 export function handleCategoricalNameChange(payload) {
     return function (dispatch, getState) {
         const dataset_id = getState().dataset.id;
-        fetch(API + '/category_name',
-            {
-                body: JSON.stringify({c: payload.name, o: payload.oldValue, n: payload.value, id: dataset_id}),
-                method: 'PUT',
-                headers: {'Content-Type': 'application/json', 'Authorization': 'Bearer ' + getIdToken()},
-            }).then(result => dispatch(_handleCategoricalNameChange(payload)));
+        setCategoryNamePromise({
+            c: payload.name,
+            o: payload.oldValue,
+            n: payload.value,
+            id: dataset_id
+        }).then(result => dispatch(_handleCategoricalNameChange(payload)));
     };
 }
 
@@ -810,6 +754,76 @@ function loadDefaultDataset() {
     };
 }
 
+function restoreSavedView(savedView) {
+    return function (dispatch, getState) {
+        if (savedView.colorScheme != null) {
+            let interpolator = getInterpolator(savedView.colorScheme);
+            if (interpolator != null) {
+                savedView.colorScheme = {
+                    name: savedView.colorScheme,
+                    value: interpolator,
+                    reversed: false // FIXME
+                };
+            }
+        }
+
+        if (savedView.datasetFilter != null) {
+            for (let key in savedView.datasetFilter) {
+                let value = savedView.datasetFilter[key];
+                if (!window.Array.isArray(value)) {
+                    value.uiValue = value.value;
+                }
+            }
+        } else {
+            savedView.datasetFilter = {};
+        }
+
+        dispatch(_setLoading(true));
+        let datasetPromise;
+        if (savedView.dataset != null) {
+            datasetPromise = dispatch(setDataset(savedView.dataset, false, false));
+        } else {
+            datasetPromise = Promise.resolve();
+        }
+        datasetPromise
+            .then(() => {
+                let dataset = getState().dataset;
+                if (savedView.embeddings) {
+                    let names = dataset.embeddings.map(e => getEmbeddingKey(e));
+                    let embeddings = [];
+                    savedView.embeddings.forEach(embedding => {
+                        let index = names.indexOf(getEmbeddingKey(embedding));
+                        if (index !== -1) {
+                            embeddings.push(dataset.embeddings[index]);
+                        }
+                    });
+                    savedView.embeddings = embeddings;
+                    if (savedView.camera != null) {
+                        savedView.chartOptions.camera = savedView.camera;
+                    }
+                }
+            })
+            .then(() => dispatch(setDatasetFilter(savedView.datasetFilter)))
+            .then(() => dispatch(restoreView(savedView)))
+            .then(() => dispatch(_updateCharts()))
+            .then(() => dispatch(handleFilterUpdated()))
+            .then(() => {
+                if (savedView.sort != null) {
+                    for (let name in savedView.sort) {
+                        dispatch(setDotPlotSortOrder({name: name, value: savedView.sort[name]}));
+                    }
+                }
+
+            })
+            .finally(() => dispatch(_setLoading(false)))
+            .catch(err => {
+                console.log(err);
+                dispatch(setMessage('Unable to restore saved view.'));
+                dispatch(loadDefaultDataset());
+            });
+    };
+}
+
 function _loadSavedView() {
     return function (dispatch, getState) {
         let savedView = {dataset: null};
@@ -823,60 +837,7 @@ function _loadSavedView() {
         }
 
         if (savedView.dataset != null) {
-            if (savedView.colorScheme != null) {
-                let interpolator = getInterpolator(savedView.colorScheme);
-                if (interpolator != null) {
-                    savedView.colorScheme = {
-                        name: savedView.colorScheme,
-                        value: interpolator,
-                        reversed: false // FIXME
-                    };
-                }
-            }
-            if (savedView.datasetFilter != null) {
-                for (let key in savedView.datasetFilter) {
-                    let value = savedView.datasetFilter[key];
-                    if (!window.Array.isArray(value)) {
-                        value.uiValue = value.value;
-                    }
-                }
-            } else {
-                savedView.datasetFilter = {};
-            }
-
-            dispatch(_setLoading(true));
-            dispatch(setDataset(savedView.dataset, false, false))
-                .then(() => {
-                    let dataset = getState().dataset;
-                    if (savedView.embeddings) {
-                        let names = dataset.embeddings.map(e => getEmbeddingKey(e));
-                        let embeddings = [];
-                        savedView.embeddings.forEach(embedding => {
-                            let index = names.indexOf(getEmbeddingKey(embedding));
-                            if (index !== -1) {
-                                embeddings.push(dataset.embeddings[index]);
-                            }
-                        });
-                        savedView.embeddings = embeddings;
-                    }
-                })
-                .then(() => dispatch(setDatasetFilter(savedView.datasetFilter)))
-                .then(() => dispatch(restoreView(savedView)))
-                .then(() => dispatch(_updateCharts()))
-                .then(() => dispatch(handleFilterUpdated()))
-                .then(() => {
-                    if (savedView.sort != null) {
-                        for (let name in savedView.sort) {
-                            dispatch(setDotPlotSortOrder({name: name, value: savedView.sort[name]}));
-                        }
-                    }
-                })
-                .finally(() => dispatch(_setLoading(false)))
-                .catch(err => {
-                    console.log(err);
-                    dispatch(setMessage('Unable to restore saved view.'));
-                    dispatch(loadDefaultDataset());
-                });
+            dispatch(restoreSavedView(savedView));
         } else {
             dispatch(loadDefaultDataset());
         }
@@ -967,7 +928,7 @@ export function saveDataset(payload) {
                 dispatch(setMessage('Dataset updated'));
             } else {
                 dispatch(_addDataset({name: name, id: importDatasetResult.id, owner: true}));
-                if(serverInfo.email) {
+                if (serverInfo.email) {
                     dispatch(setMessage(updatePermissions ? 'Please ensure that ' + serverInfo.email + ' is a "Storage Object Viewer"' : 'Dataset created'));
                 }
             }
@@ -1135,12 +1096,14 @@ export function setBinSummary(payload) {
 }
 
 export function setBinValues(payload) {
-    return function (dispatch, getState) {
-        dispatch({type: SET_BIN_VALUES, payload: payload});
-    };
+    return {type: SET_BIN_VALUES, payload: payload};
 }
 
 
+export function setSavedDatasetState(payload) {
+    return {type: SET_SAVED_DATASET_STATE, payload: payload};
+}
+
 function setDatasetFilters(payload) {
     return {type: SET_DATASET_FILTERS, payload: payload};
 }
@@ -1151,6 +1114,7 @@ export function setDataset(id, loadDefaultView = true, setLoading = true) {
         if (setLoading) {
             dispatch(_setLoading(true));
         }
+        let savedDatasetState = getState().savedDatasetState[id];
         const datasetChoices = getState().datasetChoices;
         let choice = null;
         for (let i = 0; i < datasetChoices.length; i++) {
@@ -1211,9 +1175,7 @@ export function setDataset(id, loadDefaultView = true, setLoading = true) {
             });
             newDataset.id = id;
             dispatch(_setDataset(newDataset));
-            if (loadDefaultView) {
-                dispatch(loadDefaultDatasetEmbedding());
-            }
+
             categoryNameResults.forEach(result => {
                 dispatch(_handleCategoricalNameChange({
                     name: result.category,
@@ -1222,25 +1184,23 @@ export function setDataset(id, loadDefaultView = true, setLoading = true) {
                 }));
             });
             dispatch(setDatasetFilters(datasetFilters));
+            if (savedDatasetState) {
+
+                dispatch(restoreSavedView(savedDatasetState));
+            } else if (loadDefaultView) {
+                dispatch(loadDefaultDatasetEmbedding());
+            }
         }
 
-        const categoriesRenamePromise = fetch(API + '/category_name?id=' + id,
-            {
-                headers: {'Authorization': 'Bearer ' + getIdToken()},
-            }).then(result => result.json())
+        const categoriesRenamePromise = getCategoryNamesPromise(id).then(result => result.json())
             .then(results => {
                 categoryNameResults = results;
             });
-        const filtersPromise = fetch(API + '/filters?id=' + id,
-            {
-                headers: {'Authorization': 'Bearer ' + getIdToken()},
-            }).then(result => result.json())
+        const filtersPromise = getFiltersPromise(id).then(result => result.json())
             .then(results => {
                 datasetFilters = results;
             });
-        const schemaPromise = fetch(API + '/schema?id=' + id, {headers: {'Authorization': 'Bearer ' + getIdToken()}}).then(response => {
-            return response.json();
-        }).then(result => {
+        const schemaPromise = getSchemaPromise(id).then(result => {
             newDataset = result;
         });
         return Promise.all([schemaPromise, filtersPromise, categoriesRenamePromise]).then(() => {
@@ -1434,17 +1394,11 @@ function _updateCharts(onError) {
                 }
             }
             if (measures.length > 0 || dimensions.length > 0) {
-
-                let p = fetch(API + '/embedding',
-                    {
-                        body: JSON.stringify(Object.assign({
-                            id: state.dataset.id,
-                            measures: measures,
-                            dimensions: dimensions
-                        }, getEmbeddingJson(embedding))),
-                        method: 'POST',
-                        headers: {'Content-Type': 'application/json', 'Authorization': 'Bearer ' + getIdToken()},
-                    }).then(r => r.json()).then(result => {
+                let p = getEmbeddingPromise(Object.assign({
+                    id: state.dataset.id,
+                    measures: measures,
+                    dimensions: dimensions
+                }, getEmbeddingJson(embedding))).then(result => {
                     embeddingResults.push({embeddingResult: result, embedding: embedding});
                 });
                 promises.push(p);
@@ -1475,17 +1429,11 @@ function _updateCharts(onError) {
 
 
         if (globalFeatureSummaryX.length > 0 || globalFeatureSummaryObsContinuous.length > 0 || globalFeatureSummaryDimensions.length > 0) {
-            let p = fetch(API + '/stats',
-                {
-                    body: JSON.stringify({
-                        id: state.dataset.id,
-                        measures: globalFeatureSummaryX.concat(globalFeatureSummaryObsContinuous),
-                        dimensions: globalFeatureSummaryDimensions
-                    }),
-                    method: 'POST',
-                    headers: {'Content-Type': 'application/json', 'Authorization': 'Bearer ' + getIdToken()},
-                }).then(r => r.json()).then(result => {
-
+            let p = getStatsPromise({
+                id: state.dataset.id,
+                measures: globalFeatureSummaryX.concat(globalFeatureSummaryObsContinuous),
+                dimensions: globalFeatureSummaryDimensions
+            }).then(result => {
                 const newSummary = result.summary || {};
                 for (let key in newSummary) {
                     globalFeatureSummary[key] = newSummary[key];
@@ -1534,6 +1482,7 @@ function _updateCharts(onError) {
                     });
                 }
             });
+            dotPlotData = dotPlotData.filter(categoryItem => categoryItem.values.length > 0);
             dotPlotData.sort((a, b) => {
                 a = a.name;
                 b = b.name;
@@ -1545,16 +1494,11 @@ function _updateCharts(onError) {
 
 
         if (dotplotDimensions.length > 0 && dotplotMeasures.size > 0) {
-            let p = fetch(API + '/grouped_stats',
-                {
-                    body: JSON.stringify({
-                        id: state.dataset.id,
-                        measures: Array.from(dotplotMeasures),
-                        dimensions: dotplotDimensions
-                    }),
-                    method: 'POST',
-                    headers: {'Content-Type': 'application/json', 'Authorization': 'Bearer ' + getIdToken()},
-                }).then(r => r.json()).then(dotPlotResult => {
+            let p = getGroupedStatsPromise({
+                id: state.dataset.id,
+                measures: Array.from(dotplotMeasures),
+                dimensions: dotplotDimensions
+            }).then(dotPlotResult => {
                 newDotplotData = dotPlotResult.dotplot;
             });
             promises.push(p);
@@ -1580,17 +1524,12 @@ function _updateCharts(onError) {
             // });
 
 
-            let p = fetch(API + '/selection',
-                {
-                    body: JSON.stringify({
-                        id: state.dataset.id,
-                        filter: filterJson,
-                        measures: globalFeatureSummaryDimensions.length > 0 ? searchTokens.X : globalFeatureSummaryX,
-                        dimensions: searchTokens.obsCat
-                    }),
-                    method: 'POST',
-                    headers: {'Content-Type': 'application/json', 'Authorization': 'Bearer ' + getIdToken()},
-                }).then(r => r.json()).then(result => {
+            let p = getSelectionPromise({
+                id: state.dataset.id,
+                filter: filterJson,
+                measures: globalFeatureSummaryDimensions.length > 0 ? searchTokens.X : globalFeatureSummaryX,
+                dimensions: searchTokens.obsCat
+            }).then(result => {
                 selectionResult = result;
             });
             promises.push(p);
@@ -1608,7 +1547,21 @@ function _updateCharts(onError) {
             //     b = b.name.toLowerCase();
             //     return a < b ? -1 : 1;
             // });
-
+            if (state.chartOptions.activeEmbedding != null) { // put last so that it becomes active
+                let index = -1;
+                for (let i = 0; i < embeddingData.length; i++) {
+                    if (state.chartOptions.activeEmbedding === getTraceKey(embeddingData[i])) {
+                        index = i;
+                        break;
+                    }
+                }
+                if (index !== -1) {
+                    let activeTrace = embeddingData[index];
+                    embeddingData.splice(index, 1);
+                    embeddingData.push(activeTrace);
+                }
+                state.chartOptions.activeEmbedding = null;
+            }
             dispatch(setEmbeddingData(embeddingData.slice()));
         }).finally(() => {
             dispatch(_setLoading(false));
@@ -1714,7 +1667,7 @@ function handleEmbeddingResult(result) {
                 continuous: !isCategorical,
                 isCategorical: isCategorical,
                 values: values, // for color
-                isImage:isSpatial
+                isImage: isSpatial
                 // purity: purity,
                 // text: values,
             };
@@ -1725,7 +1678,7 @@ function handleEmbeddingResult(result) {
 
             if (isSpatial) {
                 chartData.isImage = true;
-                const url = API + '/file?id=' + state.dataset.id + '&file=' + selectedEmbedding.spatial.image + '&access_token=' + getIdToken();
+                const url = getFileUrl(state.dataset.id, selectedEmbedding.spatial.image);
                 let tileSource = new OpenSeadragon.ImageTileSource({
                     url: url,
                     buildPyramid: true,
@@ -1757,10 +1710,7 @@ function handleError(dispatch, err, message) {
 export function listDatasets() {
     return function (dispatch, getState) {
         dispatch(_setLoading(true));
-        return fetch(API + '/datasets', {headers: {'Authorization': 'Bearer ' + getIdToken()}})
-            .then(response => {
-                return response.json();
-            })
+        return getDatasetsPromise()
             .then(choices => {
                 dispatch(_setDatasetChoices(choices));
             })
