@@ -3,7 +3,6 @@ import {combineReducers} from 'redux';
 import {
     ADD_DATASET,
     DELETE_DATASET,
-    DIFF_EXP_RESULTS,
     getTraceKey,
     RESTORE_VIEW,
     SET_BIN_SUMMARY,
@@ -34,6 +33,7 @@ import {
     SET_NUMBER_OF_BINS_UI,
     SET_POINT_SIZE,
     SET_PRIMARY_TRACE_KEY,
+    SET_SAVED_DATASET_STATE,
     SET_SEARCH_TOKENS,
     SET_SELECTED_DOT_PLOT_DATA,
     SET_SELECTED_EMBEDDING,
@@ -53,6 +53,14 @@ export const DEFAULT_POINT_SIZE = 1;
 export const DEFAULT_MARKER_OPACITY = 1;
 export const DEFAULT_UNSELECTED_MARKER_OPACITY = 0.1;
 export const DEFAULT_INTERPOLATOR = 'Viridis';
+export const DEFAULT_DRAG_MODE = 'pan';
+export const DEFAULT_SHOW_LABELS = false;
+export const DEFAULT_SHOW_AXIS = true;
+export const DEFAULT_SHOW_FOG = true;
+export const DEFAULT_DARK_MODE = false;
+export const DEFAULT_LABEL_FONT_SIZE = 14;
+export const DEFAULT_LABEL_STROKE_WIDTH = 4;
+
 const DEFAULT_INTERPOLATOR_OBJ = {
     name: DEFAULT_INTERPOLATOR,
     reversed: false,
@@ -60,8 +68,15 @@ const DEFAULT_INTERPOLATOR_OBJ = {
 };
 
 const DEFAULT_CHART_OPTIONS = {
-    animating: false, dragmode: 'pan', editSelection: false, showLabels: false,
-    showAxis: true, showFog: true, darkMode: false
+    animating: false,
+    dragmode: DEFAULT_DRAG_MODE,
+    editSelection: false,
+    showLabels: DEFAULT_SHOW_LABELS,
+    showAxis: DEFAULT_SHOW_AXIS,
+    showFog: DEFAULT_SHOW_FOG,
+    darkMode: DEFAULT_DARK_MODE,
+    labelFontSize: DEFAULT_LABEL_FONT_SIZE,
+    labelStrokeWidth: DEFAULT_LABEL_STROKE_WIDTH
 };
 
 function chartSize(state = 500, action) {
@@ -97,6 +112,8 @@ function chartOptions(state = DEFAULT_CHART_OPTIONS, action) {
     switch (action.type) {
         case SET_CHART_OPTIONS:
             return Object.assign({}, action.payload);
+        case RESTORE_VIEW:
+            return action.payload.chartOptions ? Object.assign(DEFAULT_CHART_OPTIONS, action.payload.chartOptions) : state;
         default:
             return state;
     }
@@ -126,17 +143,6 @@ function embeddings(state = [], action) {
     }
 }
 
-function diffExpResults(state = null, action) {
-    switch (action.type) {
-        case DIFF_EXP_RESULTS:
-            return action.payload;
-        case SET_DATASET:
-            return null;
-        default:
-            return state;
-    }
-}
-
 
 function binSummary(state = DEFAULT_BIN_SUMMARY, action) {
     switch (action.type) {
@@ -597,6 +603,17 @@ export function pointSize(state = DEFAULT_POINT_SIZE, action) {
     }
 }
 
+
+// used to restore state when toggling datasets
+export function savedDatasetState(state = {}, action) {
+    switch (action.type) {
+        case SET_SAVED_DATASET_STATE:
+            return action.payload;
+        default:
+            return state;
+    }
+}
+
 export function primaryTraceKey(state = null, action) {
     switch (action.type) {
         case SET_PRIMARY_TRACE_KEY:
@@ -648,7 +665,6 @@ export default combineReducers({
     datasetChoices,
     dialog,
     dotPlotData,
-    diffExpResults,
     email,
     embeddingData,
     embeddings,
@@ -664,6 +680,7 @@ export default combineReducers({
     message,
     pointSize,
     primaryTraceKey,
+    savedDatasetState,
     searchTokens,
     selection,
     selectedDotPlotData,
